#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "getopt.h"
#include "lutil.h"
#include "xutil.h"
#include "ftn.h"
#include "nodelist.h"
#include "config.h"
#include "version.h"

extern int verbose;
extern char *configname;

int master=0;
int forcedcalls=0;
char *forcedphone=NULL;
char *forcedline=NULL;
#ifdef HAS_TCP
char *inetaddr=NULL;
#endif
#ifdef NOISEDEBUG
int junklevel=0;
#endif

int call(faddr *);
fa_list *callall(void);
int answer(char *);
void mkdirs(char*);

void usage(name)
char *name;
{
	fprintf(stderr,"%s\tVersion %s of %s\n\t\tCopyright (c) %s\n",
		name,version,reldate,copyright);
	fprintf(stderr,"This is free software. You can do what you wish \
with it\nas long as this copyright notice is preserved.\n\n");
	fprintf(stderr,"usage: %s -h -r<role> -x<N> -I<file> <node> ...\n",name);
	fprintf(stderr,"-h		get this help\n");
	fprintf(stderr,"-r 0|1		1 - master, 0 - slave	[0]\n");
#ifdef NOISEDEBUG
	fprintf(stderr,"-j<num>		damage every <num> byte	[%d]\n",
								junklevel);
#endif
	fprintf(stderr,"-n<phone>	forced phone number\n");
	fprintf(stderr,"-l<ttydevice>	forced tty device\n");
#ifdef HAS_TCP
	fprintf(stderr,"-a<inetaddr>	use TCP/IP instead of modem\n");
#endif
	fprintf(stderr,"-x<N>		set debug level to <N>	[%d]\n",
		verbose);
	fprintf(stderr,"-I<file> 	use config file	<file>	[%s]\n",
		configname);
	fprintf(stderr,"  <node>	in domain form, e.g. f11.n22.z3\n");
	fprintf(stderr,"		(this implies master mode)\n");
	fprintf(stderr,"\n or: %s tsync|yoohoo|**EMSI_INQC816\n",name);
	fprintf(stderr,"		(this implies slave mode)\n");
}

int main(argc,argv)
int argc;
char *argv[];
{
	int c;
	fa_list *callist = NULL,*tmpl = NULL;
	faddr *tmp;
	int rc,maxrc;
	char *answermode=NULL,*p;
	int verbset=0;

#ifdef NOISEDEBUG
	while ((c=getopt(argc,argv,"j:x:r:n:l:a:I:h")) != -1)
#else
	while ((c=getopt(argc,argv,"x:r:n:l:a:I:h")) != -1)
#endif
	switch (c)
	{
		case 'x':	verbose=atoi(optarg); verbset=1; break;
#ifdef NOISEDEBUG
		case 'j':	junklevel=atoi(optarg); break;
#endif
		case 'r':	master=atoi(optarg);
				if ((master != 0) && (master != 1))
				{
					usage(argv[0]);
					exit(1);
				}
				break;
		case 'l':	forcedline=optarg; break;
#ifdef HAS_TCP
		case 'a':	inetaddr=optarg; break;
#endif
		case 'n':	forcedphone=optarg; break;
		case 'I':	configname=optarg; break;
		case 'h':
		default:	usage(argv[0]); exit(1);
	}

	if (readconfig())
	{
		fprintf(stderr,"Error getting configuration, aborting\n");
		exit(1);
	}

	if (!verbset) verbose=configverbose;

	if (debugfile) freopen(debugfile,"a",stderr);

	while (argv[optind])
	{
		for (p=argv[optind];(*p) && (*p == '*');p++);
		if ((strcasecmp(argv[optind],"tsync") == 0) ||
		    (strcasecmp(argv[optind],"yoohoo") == 0) ||
		    (strncasecmp(p,"EMSI_",5) == 0))
		{
			master=0;
			answermode=argv[optind];
			debug(5,"inbound \"%s\" mode",answermode);
		}
		else
		{
			debug(5,"callist entry \"%s\"",argv[optind]);
			if ((tmp=parsefaddr(argv[optind])))
			{
				if (callist == NULL)
				{
					callist=(fa_list *)xmalloc(sizeof(fa_list));
					tmpl=callist;
				}
				else
				{
					tmpl->next=(fa_list *)xmalloc(sizeof(fa_list));
					tmpl=tmpl->next;
				}
				tmpl->next=NULL;
				tmpl->addr=tmp;
			}
			else log("unrecognizable address \"%s\"",argv[optind]);
		}
		optind++;
	}

	if (callist)
	{
		master=1;
		forcedcalls=1;
	}

	log("start %s%s%s, role=%s, verbose=%d",argv[0],
		answermode?" ":"",
		answermode?answermode:"",
		master?"master":"slave",verbose);

	umask(066); /* packets may contain confidential information */

	p=xstrcpy(inbound);
	p=xstrcat(p,"/tmp/");
	mkdirs(p);
	free(p);

	maxrc=0;
	if (master)
	{
		if (callist == NULL) callist=callall();
		for (tmpl=callist;tmpl;tmpl=tmpl->next)
		{
			rc=call(tmpl->addr);
			log ("call to %s %s (rc=%d)",ascfnode(tmpl->addr,0x1f),
				rc?"failed":"successfull",rc);
			if (abs(rc) > maxrc) maxrc=abs(rc);
		}
		if (callist == NULL)
			log("no outbound mail");
	}
	else /* slave */
	{
		maxrc=answer(answermode);
	}

	log("end %s, rc=%d",argv[0],maxrc);

	return maxrc;
}
