#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "getopt.h"
#include "lutil.h"
#include "xutil.h"
#include "ftn.h"
#include "nodelist.h"
#include "config.h"
#include "version.h"

extern int verbose;

void usage(name)
char *name;
{
	fprintf(stderr,"%s\tVersion %s of %s\n\t\tCopyright (c) %s\n",
		name,version,reldate,copyright);
	fprintf(stderr,"This is free software. You can do what you wish \
with it\nas long as this copyright notice is preserved.\n\n");
	fprintf(stderr,"usage: %s -h -x<N> -I<file> <node>\n",name);
	fprintf(stderr,"-h		get this help\n");
	fprintf(stderr,"-x<N>		set debug level to <N>	[%d]\n",
		verbose);
	fprintf(stderr,"-I<file> 	use config file	<file>	[%s]\n",
		configname);
	fprintf(stderr,"  <node>	in domain form, e.g. f11.n22.z3\n");
}

faddr *bestroute(faddr*,faddr*,node*);

int main(argc,argv)
int argc;
char *argv[];
{
	int verbset=0;
	int c,rc;
	faddr *adr=NULL,*raddr=NULL;
	faddr *bestaka;
	node *nlent;

	while ((c=getopt(argc,argv,"x:r:n:l:a:I:h")) != -1)
	switch (c)
	{
		case 'x':	verbose=atoi(optarg); verbset=1; break;
		case 'I':	configname=optarg; break;
		case 'h':
		default:	usage(argv[0]); exit(1);
	}

	if (readconfig())
	{
		fprintf(stderr,"Error getting configuration, aborting\n");
		exit(128);
	}

	if (!verbset) verbose=configverbose;

	if (debugfile) freopen(debugfile,"a",stderr);

	if (argv[optind] == NULL)
	{
		usage(argv[0]);
		rc=128;
	}
	else if ((adr=parsefaddr(argv[optind])) == NULL)
	{
		log("unrecognizable address \"%s\"",argv[optind]);
		rc=3;
	}
	else
	{
		bestaka=bestaka_s(adr);
		nlent=getnlent(adr);
		if (nlent->pflag != DUMMY)
		{
			raddr=bestroute(adr,bestaka,nlent);
			printf("%s\n",ascinode(raddr,0x3f));
			rc=0;
		}
		else rc=1;
	}

	log("%s %s return %s, rc=%d",argv[0],argv[optind],
		ascinode(raddr,0x3f),rc);

	return rc;
}

faddr *bestroute(remote,local,nlent)
faddr *remote,*local;
node *nlent;
{
	return remote;
}
